#ifndef KREPORT_CONFIG_H
#define KREPORT_CONFIG_H

/* config-kreport.h. Generated by cmake from config-kreport.h.cmake */

/*! @file config-kreport.h
    Global KReport configuration (build time)
*/

//! @def KREPORT_GIT_SHA1_STRING
//! @brief Indicates the git sha1 commit which was used for compilation of KReport
/* #undef KREPORT_GIT_SHA1_STRING */

//! @def KREPORT_GIT_BRANCH_STRING
//! @brief Indicates the git branch which was used for compilation of KReport
/* #undef KREPORT_GIT_BRANCH_STRING */

//! @def KREPORT_SCRIPTING
//! @brief Scripting support using JavaScript language
#define KREPORT_SCRIPTING

//! @def KREPORT_UNFINISHED
//! @brief Defined if unfinished features of KReport are enabled
/* #undef KREPORT_UNFINISHED */

//! @def KREPORT_BASE_NAME
//! @brief Base name for the KReport library, based on major stable version.
//!        Useful for co-installability.
#define KREPORT_BASE_NAME "KReport3"

//! @def KREPORT_BASE_NAME_LOWER
//! @brief Like KREPORT_BASE_NAME but lowercase.
#define KREPORT_BASE_NAME_LOWER "kreport3"

//! @def KREPORT_STABLE_VERSION_MAJOR
//! @brief Extra version info, stable major, e.g. 3 for 3.1.0 Alpha (3.0.90)
#define KREPORT_STABLE_VERSION_MAJOR 3

//! @def KREPORT_STABLE_VERSION_MINOR
//! @brief Extra version info, stable minor, e.g. 1 for 3.1.0 Alpha (3.0.90)
#define KREPORT_STABLE_VERSION_MINOR 2

//! @def KREPORT_STABLE_VERSION_PATCH
//! @brief Extra version info, stable patch, e.g. 0 for 3.1.0 Alpha (3.0.90)
#define KREPORT_STABLE_VERSION_PATCH 0

//! @def KREPORT_TESTING_EXPORT
//! @brief Export symbols for testing
#ifdef BUILD_TESTING
#  define KREPORT_TESTING_EXPORT KREPORT_EXPORT
#else
#  define KREPORT_TESTING_EXPORT
#endif

#endif
